package DotForward;

require 5.004;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use DotForward ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
@EXPORT_OK = ( );

@EXPORT = qw(
	
);
$VERSION = '0.03';


# Preloaded methods go here.

sub new() {
    my $class = shift;
    my $user = shift || '';
    my $filename = shift || '';
    my $self = { 'user' => $user, 'filename' => $filename,
		 'list' => [], 'keep' => 0, @_ };

    if ( $self->{filename} ne '' && -f $self->{filename} ) {
	if ( !open(FWD, $self->{filename}) ){
	    warn "can't open $filename" unless $self->{nowarn};
	    return 0;
	}
	my ($line, $part);
	while ( $line = <FWD> ) {
	    chomp $line;
	    foreach $part ( split /\s*,\s*/, $line ) {
		$part =~ s/\s+//g;
		if ( $part =~ /^\\$self->{user}$/ ) {
		    $self->{keep} = 1;
		    next;
		}
		push @{$self->{list}}, $part;
	    }
	}
	close FWD;
    }
    
    bless $self, $class;
}

sub keep() {
    my $self = shift;
    my $set = shift;
    
    if ( defined $set  ) {
	$self->{keep} = $set;
    }
    
    $self->{keep};
}

sub list() {
    my $self = shift;
    
    if ( @_ != 0 ) {
	$self->{list} = [@_];
    }
    
    @{$self->{list}};
}

sub flush() {
    my $self = shift;
    $self->{list} = [];
}

sub add() {
    my $self = shift;
    push @{$self->{list}}, @_;
}

sub del() {
    my $self = shift;
    my ($part, $i, $delcount);
    
    $delcount = 0;
    foreach $part ( @_ ) {
	foreach $i ( 0..($#{$self->{list}}-1) ) {
	     if ( $self->{list}->[$i] eq $part ){ 
		splice(@{$self->{list}}, $i, 1);
		$delcount++;
	    }
	}
    }
    $delcount;
}

sub write() {
    my $self = shift;
    my $filename = shift;
    my $i = 0;
    
    $self->{filename} = $filename if defined $filename;

    if ( $self->{filename} eq '' ) {
	warn "write method called. bat filename is null." unless $self->{nowarn};
	return 0
    }
    
    while ( !open(FWD, '>' . $self->{filename}) ){
	sleep(3);
	if ( $i == 3 ) {
	    warn "file ", $self->{filename}, " open faild. abort." unless $self->{nowarn};
	    return 0;
	}
	$i++;
    }
    flock(FWD,2);
    
    print FWD $self->{keep} ? "\\" . $self->{user} . "\n" : '',
	join("\n", @{$self->{list}}), "\n";
    
    close FWD;
}

1;

__END__

=head1 NAME

DotForward - vague parse .forward file

=head1 SYNOPSIS

  use DotForward;
  my $fwd = DotForward->new("uid", "/home/hoge/.forward");
  $fwd->add("forward-to@nyo.com");
  $fwd->write;
  print "uid's email will be forwarded to ", $fwd->list, "\n";

=head1 DESCRIPTION

vague vague vague...
 
=head2 methods

=item * new

  DotForward->new(user_name);
  DotForward->new(user_name, filename);
  DotForward->new(user_name, filename, nowarn => 1 );

user_name for keep method.
if set filename, read it.
  
=item * list

  DotForward->list();
  DotForward->list('a@hoge.com', 'b@hoge.com');

Retrun forward address list.
If set argument, replace address list.
  
=item * add

  DotForward->add('c@hoge.com', 'd@hoge.com');

Add address to forward list.
  
=item * del

  DotForward->del('c@hoge.com', 'd@hoge.com');

Delete address from forward list.
Return num of delete address.

=item * flush

  DotForward->flush();

Delete ALL address.

=item * keep

  DotForward->keep();
  DotForward->keep(0);
  DotForward->keep(1);

Retrun 0 or 1.
If set argument, set it.

=item * write

  DotForward->write();
  DotForward->write(filename);

Write data to file.
If set filename, write to that file.

=head2 EXPORT

None.

=head1 AUTHOR

Tatsuki Sugiura <sugi@nemui.org>

=head1 SEE ALSO

perl(1).

=cut
